"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCache = exports.isRecordExpired = void 0;
const bottleneck_1 = __importDefault(require("bottleneck"));
const type_fns_1 = require("type-fns");
const updateValidKeysBottleneck = new bottleneck_1.default({ maxConcurrent: 1 });
/**
 * the key under which to persist the valid_keys metadata
 */
const RESERVED_CACHE_KEY_FOR_VALID_KEYS = 'keys';
const getMseNow = () => new Date().getTime();
/**
 * a utility function for deciding whether a record is valid
 */
const isRecordExpired = ({ expiresAtMse, }) => {
    // if expiresAtMse = null, then it never expires
    if (expiresAtMse === null)
        return false;
    // otherwise, check whether its expired
    return expiresAtMse < getMseNow();
};
exports.isRecordExpired = isRecordExpired;
const createCache = ({ namespace, defaultSecondsUntilExpiration = 5 * 60, }) => {
    // define how to get the full cache key
    const getCacheKey = (key) => ['cache', namespace, key].join(':');
    // define how to set an item into the cache
    const set = (key, value, { secondsUntilExpiration = defaultSecondsUntilExpiration, } = {}) => {
        const cacheKey = getCacheKey(key);
        // handle cache invalidation
        if (value === undefined) {
            window.localStorage.removeItem(cacheKey);
            return { key, expiresAtMse: 0 };
        }
        // handle setting
        const expiresAtMse = secondsUntilExpiration
            ? getMseNow() + secondsUntilExpiration * 1000
            : null;
        const serializedValue = JSON.stringify({
            expiresAtMse,
            value,
        });
        window.localStorage.setItem(cacheKey, serializedValue);
        // return the key with metadata
        return {
            key,
            expiresAtMse,
        };
    };
    // define how to get an item from the cache
    const get = (key) => {
        const cacheKey = getCacheKey(key);
        const cacheContentJSON = window.localStorage.getItem(cacheKey);
        if (!cacheContentJSON)
            return undefined; // if not in cache, then undefined
        const cacheContent = JSON.parse(cacheContentJSON);
        if ((0, exports.isRecordExpired)(cacheContent))
            return undefined; // if already expired, then undefined
        return cacheContent.value; // otherwise, its in the cache and not expired, so return the value
    };
    /**
     * define how to lookup valid keys for the cache
     */
    const getValidKeysWithMetadata = () => __awaiter(void 0, void 0, void 0, function* () {
        // lookup the last saved valid keys
        const cachedKeyMetadataJSON = yield get(RESERVED_CACHE_KEY_FOR_VALID_KEYS);
        const cachedKeyMetadata = cachedKeyMetadataJSON
            ? JSON.parse(cachedKeyMetadataJSON)
            : [];
        // purge expired keys
        const cachedExpiredKeys = cachedKeyMetadata.filter(exports.isRecordExpired);
        cachedExpiredKeys.map(({ key }) => set(key, undefined));
        // return the valid keys
        const validKeyMetadata = cachedKeyMetadata.filter((0, type_fns_1.withNot)(exports.isRecordExpired));
        return validKeyMetadata;
    });
    /**
     * define how to save valid keys for the cache
     *
     * note
     * - record a key w/ effectiveAtMse = 0 to invalidate it
     *
     * TODO: eventually, support lossless high-concurrency writing (potentially optionally, as a cache option, since it's not important for most applications)
     * - we need some way of ensuring that parallel processes wont conflict + overwrite eachother
     *   - for example, imagine you have two keys that were set to cache in parallel
     *     - requestA = [...savedKeys, newKeyA]
     *     - requestB = [...savedKeys, newKeyB]
     *     - read-before-write would make it so that either newKeyA or newKeyB is dropped and doesn't make it to the final destination // TODO: lookup the formal word for this race condition, its common in dbs
     *   - in other words,
     *     - there is a risk a query _will_ have been cached but not saved to the valid keys -> immediately invalidated
     *     - this is a safe failure mode, as it's the same as the query never having been cached in the first place (i.e., just requires extra requests)
     *   - if we find a usecase where it _is_ critical to solve, we can do so
     *     - probably with
     *       - per-thread "append" file (which all read from, but only one thread writes to)  (similar in spi)
     *       - plus
     *       - globally locked global file update, similar to
     *       - inspiration: https://stackoverflow.com/a/53193851/3068233
     */
    const updateKeyWithMetadataState = ({ for: forKeyWithMetadata, }) => __awaiter(void 0, void 0, void 0, function* () {
        // write inside of a bottleneck, to ensure that within one runtime no more than one process is writing to that same key; prevents same-runtime race conditions
        return updateValidKeysBottleneck.schedule(() => __awaiter(void 0, void 0, void 0, function* () {
            // lookup current valid keys
            const currentKeysWithMetadata = yield getValidKeysWithMetadata();
            // save the keys w/ an extra key
            yield set(RESERVED_CACHE_KEY_FOR_VALID_KEYS, JSON.stringify([
                // save the current keys, excluding the previous state of this key if it was there
                ...currentKeysWithMetadata.filter(({ key }) => key !== forKeyWithMetadata.key),
                // save this key, if it isn't expired
                ...((0, exports.isRecordExpired)(forKeyWithMetadata) ? [] : [forKeyWithMetadata]),
            ]), { secondsUntilExpiration: null });
        }));
    });
    /**
     * define how to set an item to the cache, with valid key tracking
     */
    const setWithValidKeyTracking = (...args) => __awaiter(void 0, void 0, void 0, function* () {
        // write to the cache
        const newKeyWithMetadata = yield set(...args);
        // add the key as valid
        yield updateKeyWithMetadataState({ for: newKeyWithMetadata });
    });
    /**
     * define how to get valid keys
     */
    const getValidKeys = () => __awaiter(void 0, void 0, void 0, function* () {
        const keysWithMetadata = yield getValidKeysWithMetadata();
        return keysWithMetadata.map(({ key }) => key);
    });
    /**
     * define how to get an item from the cache, synced with valid key tracking
     */
    const getWithValidKeyTracking = (...args) => __awaiter(void 0, void 0, void 0, function* () {
        // if its not a valid key, then dont try to get (this is critical, as it ensures that the validKeys array is a source of truth)
        const validKeys = yield getValidKeys();
        if (!validKeys.includes(args[0]))
            return undefined; // if the key is not valid, then no value
        // otherwise, lookup the value
        return get(...args);
    });
    /**
     * return the api
     */
    return {
        set: setWithValidKeyTracking,
        get: getWithValidKeyTracking,
        keys: getValidKeys,
    };
};
exports.createCache = createCache;
//# sourceMappingURL=cache.js.map